package org.apache.commons.lang3.concurrent;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.concurrent.BackgroundInitializer;
import org.apache.commons.lang3.concurrent.ConcurrentException;
import org.apache.commons.lang3.concurrent.MultiBackgroundInitializer;

public class MultiBackgroundInitializer extends BackgroundInitializer<MultiBackgroundInitializer.MultiBackgroundInitializerResults> {
  private final Map<String, BackgroundInitializer<?>> childInitializers = new HashMap<>();
  
  public MultiBackgroundInitializer(ExecutorService exec) {
    super(exec);
  }
  
  public void addInitializer(String name, BackgroundInitializer<?> backgroundInitializer) {
    Validate.notNull(name, "name", new Object[0]);
    Validate.notNull(backgroundInitializer, "backgroundInitializer", new Object[0]);
    synchronized (this) {
      if (isStarted())
        throw new IllegalStateException("addInitializer() must not be called after start()!"); 
      this.childInitializers.put(name, backgroundInitializer);
    } 
  }
  
  protected int getTaskCount() {
    int result = 1;
    for (BackgroundInitializer<?> bi : this.childInitializers.values())
      result += bi.getTaskCount(); 
    return result;
  }
  
  protected MultiBackgroundInitializerResults initialize() throws Exception {
    Map<String, BackgroundInitializer<?>> inits;
    synchronized (this) {
      inits = new HashMap<>(this.childInitializers);
    } 
    ExecutorService exec = getActiveExecutor();
    for (BackgroundInitializer<?> bi : inits.values()) {
      if (bi.getExternalExecutor() == null)
        bi.setExternalExecutor(exec); 
      bi.start();
    } 
    Map<String, Object> results = new HashMap<>();
    Map<String, ConcurrentException> excepts = new HashMap<>();
    for (Map.Entry<String, BackgroundInitializer<?>> e : inits.entrySet()) {
      try {
        results.put(e.getKey(), ((BackgroundInitializer)e.getValue()).get());
      } catch (ConcurrentException cex) {
        excepts.put(e.getKey(), cex);
      } 
    } 
    return new MultiBackgroundInitializerResults(inits, results, excepts, null);
  }
  
  public MultiBackgroundInitializer() {}
}
